require_relative 'player'
require_relative 'game_turn'
require_relative 'treasure_trove'
require 'csv'

module StudioGame
    class Game
        attr_reader :title

        def initialize(title)
            @title = title
            @players = []
        end

        def add_player(player)
            @players << player
        end

        def load_players(from_file)
            # File.readlines(from_file).each do |line|
            #     add_player(Player.from_csv(line))
            # end
            CSV.foreach(from_file) do |row|
                add_player(Player.new(row[0], Integer(row[1])))
            end
        end

        def high_scores_message
            return_string = "#{@title}'s High Scores:\n"
            @players.sort.each do |player|
                formatted_name = player.name.ljust(20, '.')
                return_string << "#{formatted_name} #{player.score}\n"
            end
            return_string
        end

        def save_high_scores(to_file = "high_scores.txt")
            File.open(to_file, "w") do |file|
                file.puts high_scores_message
            end
        end

        def play(rounds)
            treasures = TreasureTrove::TREASURES

            puts "There are #{treasures.size} treasures to be found:"
            treasures.each do |treasure|
                puts "A #{treasure.name} is worth #{treasure.points} points"
            end

            puts "\nThere are #{@players.size} players in #{@title}:"
            @players.each do |player|
                puts player
            end

            1.upto(rounds) do |round|
                if block_given?
                    break if yield
                end
                puts "\nRound #{round}:"
                @players.each do |player|
                    GameTurn.take_turn(player)
                    puts player
                end
            end
        end

        def total_points
            @players.reduce(0) do |sum, player|
                sum + player.points
            end
        end

        def print_name_and_health(player)
            puts "#{player.name} (#{player.health})"
        end

        def print_stats
            puts "#{@title}'s Statistics:"

            puts "\n#{total_points} total points from treasures found"

            @players.sort.each do |player|
                puts "\n#{player.name}'s points totals:"
                player.each_found_treasure do |treasure|
                    puts "#{treasure.points} total #{treasure.name} points"
                end
                puts "#{player.points} grand total points"
            end

            strong_players, wimpy_players = @players.partition { |p| p.strong? }

            puts "\n#{strong_players.length} strong players:"
            strong_players.each do |p|
                print_name_and_health(p)
            end

            puts "\n#{wimpy_players.length} wimpy players:"
            wimpy_players.each do |p|
                print_name_and_health(p)
            end

            sorted_players = @players.sort { |a, b| b.score <=> a.score }

            puts high_scores_message
        end
    end
end
