require_relative 'playable'
require_relative 'treasure_trove'

module StudioGame
    class Player
        include Playable

        attr_reader :name
        attr_accessor :health

        def initialize(name, health=100)
            @name = name.capitalize
            @health = health
            @found_treasure = Hash.new(0)
        end

        # class method
        def self.from_csv(line)
            name, health = line.split(',')
            Player.new(name, Integer(health))
        end

        def found_treasure(treasure)
            @found_treasure[treasure.name] += treasure.points
            puts "#{@name} found a #{treasure.name} worth #{treasure.points} points."
            puts "#{@name}'s treasures: #{@found_treasure}"
        end

        def each_found_treasure
            @found_treasure.each do |name, points|
                treasure = Treasure.new(name, points)
                yield(treasure)
            end
        end

        def points
            @found_treasure.values.reduce(0, :+)
        end

        def score
            @health + points
        end

        def name=(new_name)
            @name = new_name.capitalize
        end

        def to_s
            "I'm #{@name} with health = #{@health}, points = #{points}, and score = #{score}."
        end

        def <=>(other_player)
            other_player.score <=> score
        end
    end

    if __FILE__ == $0
        player = Player.new("moe")
        puts player.name
        puts player.health
        player.w00t
        puts player.health
        player.blam
        puts player.health
    end
end
